# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: collections.proto, collections_service.proto, json_with_int.proto, points.proto, points_service.proto, nnext.proto, snapshots_service.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from datetime import datetime
from typing import Dict, List, Optional

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class Distance(betterproto.Enum):
    UnknownDistance = 0
    Cosine = 1
    Euclid = 2
    Dot = 3


class CollectionStatus(betterproto.Enum):
    UnknownCollectionStatus = 0
    Green = 1
    Yellow = 2
    Red = 3


class PayloadSchemaType(betterproto.Enum):
    UnknownType = 0
    Keyword = 1
    Integer = 2
    Float = 3
    Geo = 4


class NullValue(betterproto.Enum):
    """
    `NullValue` is a singleton enumeration to represent the null value for the
    `Value` type union.  The JSON representation for `NullValue` is JSON
    `null`.
    """

    # Null value.
    NULL_VALUE = 0


class FieldType(betterproto.Enum):
    FieldTypeKeyword = 0
    FieldTypeInteger = 1
    FieldTypeFloat = 2
    FieldTypeGeo = 3


class UpdateStatus(betterproto.Enum):
    UnknownUpdateStatus = 0
    Acknowledged = 1
    Completed = 2


@dataclass(eq=False, repr=False)
class GetCollectionInfoRequest(betterproto.Message):
    collection_name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ListCollectionsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CollectionDescription(betterproto.Message):
    name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetCollectionInfoResponse(betterproto.Message):
    result: "CollectionInfo" = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class ListCollectionsResponse(betterproto.Message):
    collections: List["CollectionDescription"] = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class OptimizerStatus(betterproto.Message):
    ok: bool = betterproto.bool_field(1)
    error: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class HnswConfigDiff(betterproto.Message):
    # Number of edges per node in the index graph. Larger the value - more
    # accurate the search, more space required.
    m: Optional[int] = betterproto.uint64_field(1, optional=True, group="_m")
    # Number of neighbours to consider during the index building. Larger the
    # value - more accurate the search, more time required to build index.
    ef_construct: Optional[int] = betterproto.uint64_field(
        2, optional=True, group="_ef_construct"
    )
    # Minimal size (in KiloBytes) of vectors for additional payload-based
    # indexing.If payload chunk is smaller than `full_scan_threshold` additional
    # indexing won't be used -in this case full-scan search should be preferred
    # by query planner and additional indexing is not required.Note: 1Kb = 1
    # vector of size 256
    full_scan_threshold: Optional[int] = betterproto.uint64_field(
        3, optional=True, group="_full_scan_threshold"
    )
    # Number of parallel threads used for background index building. If 0 - auto
    # selection.
    max_indexing_threads: Optional[int] = betterproto.uint64_field(
        4, optional=True, group="_max_indexing_threads"
    )


@dataclass(eq=False, repr=False)
class WalConfigDiff(betterproto.Message):
    wal_capacity_mb: Optional[int] = betterproto.uint64_field(
        1, optional=True, group="_wal_capacity_mb"
    )
    wal_segments_ahead: Optional[int] = betterproto.uint64_field(
        2, optional=True, group="_wal_segments_ahead"
    )


@dataclass(eq=False, repr=False)
class OptimizersConfigDiff(betterproto.Message):
    # The minimal fraction of deleted vectors in a segment, required to perform
    # segment optimization
    deleted_threshold: Optional[float] = betterproto.double_field(
        1, optional=True, group="_deleted_threshold"
    )
    # The minimal number of vectors in a segment, required to perform segment
    # optimization
    vacuum_min_vector_number: Optional[int] = betterproto.uint64_field(
        2, optional=True, group="_vacuum_min_vector_number"
    )
    # Target amount of segments optimizer will try to keep.Real amount of
    # segments may vary depending on multiple parameters:- Amount of stored
    # points.- Current write RPS.It is recommended to select default number of
    # segments as a factor of the number of search threads,so that each segment
    # would be handled evenly by one of the threads.
    default_segment_number: Optional[int] = betterproto.uint64_field(
        3, optional=True, group="_default_segment_number"
    )
    # Do not create segments larger this size (in KiloBytes).Large segments might
    # require disproportionately long indexation times,therefore it makes sense
    # to limit the size of segments.If indexation speed have more priority for
    # your - make this parameter lower.If search speed is more important - make
    # this parameter higher.Note: 1Kb = 1 vector of size 256
    max_segment_size: Optional[int] = betterproto.uint64_field(
        4, optional=True, group="_max_segment_size"
    )
    # Maximum size (in KiloBytes) of vectors to store in-memory per
    # segment.Segments larger than this threshold will be stored as read-only
    # memmaped file.To enable memmap storage, lower the thresholdNote: 1Kb = 1
    # vector of size 256
    memmap_threshold: Optional[int] = betterproto.uint64_field(
        5, optional=True, group="_memmap_threshold"
    )
    # Maximum size (in KiloBytes) of vectors allowed for plain index.Default
    # value based on https://github.com/google-research/google-
    # research/blob/master/scann/docs/algorithms.mdNote: 1Kb = 1 vector of size
    # 256
    indexing_threshold: Optional[int] = betterproto.uint64_field(
        6, optional=True, group="_indexing_threshold"
    )
    # Interval between forced flushes.
    flush_interval_sec: Optional[int] = betterproto.uint64_field(
        7, optional=True, group="_flush_interval_sec"
    )
    # Max number of threads, which can be used for optimization. If 0 - `NUM_CPU
    # - 1` will be used
    max_optimization_threads: Optional[int] = betterproto.uint64_field(
        8, optional=True, group="_max_optimization_threads"
    )


@dataclass(eq=False, repr=False)
class CreateCollection(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    vector_size: int = betterproto.uint64_field(2)
    distance: "Distance" = betterproto.enum_field(3)
    hnsw_config: Optional["HnswConfigDiff"] = betterproto.message_field(
        4, optional=True, group="_hnsw_config"
    )
    wal_config: Optional["WalConfigDiff"] = betterproto.message_field(
        5, optional=True, group="_wal_config"
    )
    optimizers_config: Optional["OptimizersConfigDiff"] = betterproto.message_field(
        6, optional=True, group="_optimizers_config"
    )
    shard_number: Optional[int] = betterproto.uint32_field(
        7, optional=True, group="_shard_number"
    )
    on_disk_payload: Optional[bool] = betterproto.bool_field(
        8, optional=True, group="_on_disk_payload"
    )
    timeout: Optional[int] = betterproto.uint64_field(
        9, optional=True, group="_timeout"
    )


@dataclass(eq=False, repr=False)
class UpdateCollection(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    optimizers_config: Optional["OptimizersConfigDiff"] = betterproto.message_field(
        2, optional=True, group="_optimizers_config"
    )
    timeout: Optional[int] = betterproto.uint64_field(
        3, optional=True, group="_timeout"
    )


@dataclass(eq=False, repr=False)
class DeleteCollection(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    timeout: Optional[int] = betterproto.uint64_field(
        2, optional=True, group="_timeout"
    )


@dataclass(eq=False, repr=False)
class CollectionOperationResponse(betterproto.Message):
    result: bool = betterproto.bool_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class CollectionParams(betterproto.Message):
    vector_size: int = betterproto.uint64_field(1)
    distance: "Distance" = betterproto.enum_field(2)
    shard_number: int = betterproto.uint32_field(3)
    on_disk_payload: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CollectionConfig(betterproto.Message):
    params: "CollectionParams" = betterproto.message_field(1)
    hnsw_config: "HnswConfigDiff" = betterproto.message_field(2)
    optimizer_config: "OptimizersConfigDiff" = betterproto.message_field(3)
    wal_config: "WalConfigDiff" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class PayloadSchemaInfo(betterproto.Message):
    data_type: "PayloadSchemaType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class CollectionInfo(betterproto.Message):
    status: "CollectionStatus" = betterproto.enum_field(1)
    optimizer_status: "OptimizerStatus" = betterproto.message_field(2)
    vectors_count: int = betterproto.uint64_field(3)
    segments_count: int = betterproto.uint64_field(4)
    disk_data_size: int = betterproto.uint64_field(5)
    ram_data_size: int = betterproto.uint64_field(6)
    config: "CollectionConfig" = betterproto.message_field(7)
    payload_schema: Dict[str, "PayloadSchemaInfo"] = betterproto.map_field(
        8, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    points_count: int = betterproto.uint64_field(9)
    indexed_vectors_count: Optional[int] = betterproto.uint64_field(
        10, optional=True, group="_indexed_vectors_count"
    )

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.disk_data_size:
            warnings.warn(
                "CollectionInfo.disk_data_size is deprecated", DeprecationWarning
            )
        if self.ram_data_size:
            warnings.warn(
                "CollectionInfo.ram_data_size is deprecated", DeprecationWarning
            )


@dataclass(eq=False, repr=False)
class ChangeAliases(betterproto.Message):
    actions: List["AliasOperations"] = betterproto.message_field(1)
    timeout: Optional[int] = betterproto.uint64_field(
        2, optional=True, group="_timeout"
    )


@dataclass(eq=False, repr=False)
class AliasOperations(betterproto.Message):
    create_alias: "CreateAlias" = betterproto.message_field(1, group="action")
    rename_alias: "RenameAlias" = betterproto.message_field(2, group="action")
    delete_alias: "DeleteAlias" = betterproto.message_field(3, group="action")


@dataclass(eq=False, repr=False)
class CreateAlias(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    alias_name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RenameAlias(betterproto.Message):
    old_alias_name: str = betterproto.string_field(1)
    new_alias_name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class DeleteAlias(betterproto.Message):
    alias_name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class Struct(betterproto.Message):
    """
    `Struct` represents a structured data value, consisting of fields which map
    to dynamically typed values. In some languages, `Struct` might be supported
    by a native representation. For example, in scripting languages like JS a
    struct is represented as an object. The details of that representation are
    described together with the proto support for the language. The JSON
    representation for `Struct` is JSON object.
    """

    # Unordered map of dynamically typed values.
    fields: Dict[str, "Value"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass(eq=False, repr=False)
class Value(betterproto.Message):
    """
    `Value` represents a dynamically typed value which can be either null, a
    number, a string, a boolean, a recursive struct value, or a list of values.
    A producer of value is expected to set one of that variants, absence of any
    variant indicates an error. The JSON representation for `Value` is JSON
    value.
    """

    # Represents a null value.
    null_value: "NullValue" = betterproto.enum_field(1, group="kind")
    # Represents a double value.
    double_value: float = betterproto.double_field(2, group="kind")
    # Represents an integer value
    integer_value: int = betterproto.int64_field(3, group="kind")
    # Represents a string value.
    string_value: str = betterproto.string_field(4, group="kind")
    # Represents a boolean value.
    bool_value: bool = betterproto.bool_field(5, group="kind")
    # Represents a structured value.
    struct_value: "Struct" = betterproto.message_field(6, group="kind")
    # Represents a repeated `Value`.
    list_value: "ListValue" = betterproto.message_field(7, group="kind")


@dataclass(eq=False, repr=False)
class ListValue(betterproto.Message):
    """
    `ListValue` is a wrapper around a repeated field of values. The JSON
    representation for `ListValue` is JSON array.
    """

    # Repeated field of dynamically typed values.
    values: List["Value"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PointId(betterproto.Message):
    num: int = betterproto.uint64_field(1, group="point_id_options")
    uuid: str = betterproto.string_field(2, group="point_id_options")


@dataclass(eq=False, repr=False)
class UpsertPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    wait: Optional[bool] = betterproto.bool_field(2, optional=True, group="_wait")
    points: List["PointStruct"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class DeletePoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    wait: Optional[bool] = betterproto.bool_field(2, optional=True, group="_wait")
    points: "PointsSelector" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class GetPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    ids: List["PointId"] = betterproto.message_field(2)
    with_vector: Optional[bool] = betterproto.bool_field(
        3, optional=True, group="_with_vector"
    )
    with_payload: "WithPayloadSelector" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SetPayloadPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    wait: Optional[bool] = betterproto.bool_field(2, optional=True, group="_wait")
    payload: Dict[str, "Value"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    points: List["PointId"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class DeletePayloadPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    wait: Optional[bool] = betterproto.bool_field(2, optional=True, group="_wait")
    keys: List[str] = betterproto.string_field(3)
    points: List["PointId"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ClearPayloadPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    wait: Optional[bool] = betterproto.bool_field(2, optional=True, group="_wait")
    points: "PointsSelector" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CreateFieldIndexCollection(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    wait: Optional[bool] = betterproto.bool_field(2, optional=True, group="_wait")
    field_name: str = betterproto.string_field(3)
    field_type: Optional["FieldType"] = betterproto.enum_field(
        4, optional=True, group="_field_type"
    )


@dataclass(eq=False, repr=False)
class DeleteFieldIndexCollection(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    wait: Optional[bool] = betterproto.bool_field(2, optional=True, group="_wait")
    field_name: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class PayloadIncludeSelector(betterproto.Message):
    fields: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class PayloadExcludeSelector(betterproto.Message):
    fields: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class WithPayloadSelector(betterproto.Message):
    enable: bool = betterproto.bool_field(1, group="selector_options")
    include: "PayloadIncludeSelector" = betterproto.message_field(
        2, group="selector_options"
    )
    exclude: "PayloadExcludeSelector" = betterproto.message_field(
        3, group="selector_options"
    )


@dataclass(eq=False, repr=False)
class SearchParams(betterproto.Message):
    # Params relevant to HNSW index. Size of the beam in a beam-search.Larger the
    # value - more accurate the result, more time required for search.
    hnsw_ef: Optional[int] = betterproto.uint64_field(
        1, optional=True, group="_hnsw_ef"
    )


@dataclass(eq=False, repr=False)
class SearchPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    vector: List[float] = betterproto.float_field(2)
    filter: "Filter" = betterproto.message_field(3)
    limit: int = betterproto.uint64_field(4)
    with_vector: Optional[bool] = betterproto.bool_field(
        5, optional=True, group="_with_vector"
    )
    with_payload: "WithPayloadSelector" = betterproto.message_field(6)
    params: "SearchParams" = betterproto.message_field(7)
    score_threshold: Optional[float] = betterproto.float_field(
        8, optional=True, group="_score_threshold"
    )
    offset: Optional[int] = betterproto.uint64_field(9, optional=True, group="_offset")


@dataclass(eq=False, repr=False)
class ScrollPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    filter: "Filter" = betterproto.message_field(2)
    offset: Optional["PointId"] = betterproto.message_field(
        3, optional=True, group="_offset"
    )
    limit: Optional[int] = betterproto.uint32_field(4, optional=True, group="_limit")
    with_vector: Optional[bool] = betterproto.bool_field(
        5, optional=True, group="_with_vector"
    )
    with_payload: "WithPayloadSelector" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class RecommendPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    positive: List["PointId"] = betterproto.message_field(2)
    negative: List["PointId"] = betterproto.message_field(3)
    filter: "Filter" = betterproto.message_field(4)
    limit: int = betterproto.uint64_field(5)
    with_vector: Optional[bool] = betterproto.bool_field(
        6, optional=True, group="_with_vector"
    )
    with_payload: "WithPayloadSelector" = betterproto.message_field(7)
    params: "SearchParams" = betterproto.message_field(8)
    score_threshold: Optional[float] = betterproto.float_field(
        9, optional=True, group="_score_threshold"
    )
    offset: Optional[int] = betterproto.uint64_field(10, optional=True, group="_offset")


@dataclass(eq=False, repr=False)
class CountPoints(betterproto.Message):
    collection_name: str = betterproto.string_field(1)
    filter: "Filter" = betterproto.message_field(2)
    exact: Optional[bool] = betterproto.bool_field(3, optional=True, group="_exact")


@dataclass(eq=False, repr=False)
class PointsOperationResponse(betterproto.Message):
    result: "UpdateResult" = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class UpdateResult(betterproto.Message):
    operation_id: int = betterproto.uint64_field(1)
    status: "UpdateStatus" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ScoredPoint(betterproto.Message):
    id: "PointId" = betterproto.message_field(1)
    payload: Dict[str, "Value"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    score: float = betterproto.float_field(3)
    vector: List[float] = betterproto.float_field(4)
    version: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class SearchResponse(betterproto.Message):
    result: List["ScoredPoint"] = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class CountResponse(betterproto.Message):
    result: "CountResult" = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class ScrollResponse(betterproto.Message):
    next_page_offset: Optional["PointId"] = betterproto.message_field(
        1, optional=True, group="_next_page_offset"
    )
    result: List["RetrievedPoint"] = betterproto.message_field(2)
    time: float = betterproto.double_field(3)


@dataclass(eq=False, repr=False)
class CountResult(betterproto.Message):
    count: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class RetrievedPoint(betterproto.Message):
    id: "PointId" = betterproto.message_field(1)
    payload: Dict[str, "Value"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    vector: List[float] = betterproto.float_field(3)


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    result: List["RetrievedPoint"] = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class RecommendResponse(betterproto.Message):
    result: List["ScoredPoint"] = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class Filter(betterproto.Message):
    should: List["Condition"] = betterproto.message_field(1)
    must: List["Condition"] = betterproto.message_field(2)
    must_not: List["Condition"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Condition(betterproto.Message):
    field: "FieldCondition" = betterproto.message_field(1, group="condition_one_of")
    is_empty: "IsEmptyCondition" = betterproto.message_field(
        2, group="condition_one_of"
    )
    has_id: "HasIdCondition" = betterproto.message_field(3, group="condition_one_of")
    filter: "Filter" = betterproto.message_field(4, group="condition_one_of")


@dataclass(eq=False, repr=False)
class IsEmptyCondition(betterproto.Message):
    key: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class HasIdCondition(betterproto.Message):
    has_id: List["PointId"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class FieldCondition(betterproto.Message):
    key: str = betterproto.string_field(1)
    match: "Match" = betterproto.message_field(2)
    range: "Range" = betterproto.message_field(3)
    geo_bounding_box: "GeoBoundingBox" = betterproto.message_field(4)
    geo_radius: "GeoRadius" = betterproto.message_field(5)
    values_count: "ValuesCount" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class Match(betterproto.Message):
    keyword: str = betterproto.string_field(1, group="match_value")
    integer: int = betterproto.int64_field(2, group="match_value")
    boolean: bool = betterproto.bool_field(3, group="match_value")


@dataclass(eq=False, repr=False)
class Range(betterproto.Message):
    lt: Optional[float] = betterproto.double_field(1, optional=True, group="_lt")
    gt: Optional[float] = betterproto.double_field(2, optional=True, group="_gt")
    gte: Optional[float] = betterproto.double_field(3, optional=True, group="_gte")
    lte: Optional[float] = betterproto.double_field(4, optional=True, group="_lte")


@dataclass(eq=False, repr=False)
class GeoBoundingBox(betterproto.Message):
    top_left: "GeoPoint" = betterproto.message_field(1)
    bottom_right: "GeoPoint" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GeoRadius(betterproto.Message):
    center: "GeoPoint" = betterproto.message_field(1)
    radius: float = betterproto.float_field(2)


@dataclass(eq=False, repr=False)
class ValuesCount(betterproto.Message):
    lt: Optional[int] = betterproto.uint64_field(1, optional=True, group="_lt")
    gt: Optional[int] = betterproto.uint64_field(2, optional=True, group="_gt")
    gte: Optional[int] = betterproto.uint64_field(3, optional=True, group="_gte")
    lte: Optional[int] = betterproto.uint64_field(4, optional=True, group="_lte")


@dataclass(eq=False, repr=False)
class PointsSelector(betterproto.Message):
    points: "PointsIdsList" = betterproto.message_field(
        1, group="points_selector_one_of"
    )
    filter: "Filter" = betterproto.message_field(2, group="points_selector_one_of")


@dataclass(eq=False, repr=False)
class PointsIdsList(betterproto.Message):
    ids: List["PointId"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PointStruct(betterproto.Message):
    id: "PointId" = betterproto.message_field(1)
    vector: List[float] = betterproto.float_field(2)
    payload: Dict[str, "Value"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass(eq=False, repr=False)
class GeoPoint(betterproto.Message):
    lon: float = betterproto.double_field(1)
    lat: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class CreateFullSnapshotRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListFullSnapshotsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CreateSnapshotRequest(betterproto.Message):
    collection_name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ListSnapshotsRequest(betterproto.Message):
    collection_name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class SnapshotDescription(betterproto.Message):
    name: str = betterproto.string_field(1)
    creation_time: datetime = betterproto.message_field(2)
    size: int = betterproto.int64_field(3)


@dataclass(eq=False, repr=False)
class CreateSnapshotResponse(betterproto.Message):
    snapshot_description: "SnapshotDescription" = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class ListSnapshotsResponse(betterproto.Message):
    snapshot_descriptions: List["SnapshotDescription"] = betterproto.message_field(1)
    time: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class HealthCheckRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class HealthCheckReply(betterproto.Message):
    title: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)


class CollectionsStub(betterproto.ServiceStub):
    async def get(self, *, collection_name: str = "") -> "GetCollectionInfoResponse":

        request = GetCollectionInfoRequest()
        request.collection_name = collection_name

        return await self._unary_unary(
            "/nnext.Collections/Get", request, GetCollectionInfoResponse
        )

    async def list(self) -> "ListCollectionsResponse":

        request = ListCollectionsRequest()

        return await self._unary_unary(
            "/nnext.Collections/List", request, ListCollectionsResponse
        )

    async def create(
        self,
        *,
        collection_name: str = "",
        vector_size: int = 0,
        distance: "Distance" = 0,
        hnsw_config: Optional["HnswConfigDiff"] = None,
        wal_config: Optional["WalConfigDiff"] = None,
        optimizers_config: Optional["OptimizersConfigDiff"] = None,
        shard_number: Optional[int] = None,
        on_disk_payload: Optional[bool] = None,
        timeout: Optional[int] = None
    ) -> "CollectionOperationResponse":

        request = CreateCollection()
        request.collection_name = collection_name
        request.vector_size = vector_size
        request.distance = distance
        if hnsw_config is not None:
            request.hnsw_config = hnsw_config
        if wal_config is not None:
            request.wal_config = wal_config
        if optimizers_config is not None:
            request.optimizers_config = optimizers_config
        request.shard_number = shard_number
        request.on_disk_payload = on_disk_payload
        request.timeout = timeout

        return await self._unary_unary(
            "/nnext.Collections/Create", request, CollectionOperationResponse
        )

    async def update(
        self,
        *,
        collection_name: str = "",
        optimizers_config: Optional["OptimizersConfigDiff"] = None,
        timeout: Optional[int] = None
    ) -> "CollectionOperationResponse":

        request = UpdateCollection()
        request.collection_name = collection_name
        if optimizers_config is not None:
            request.optimizers_config = optimizers_config
        request.timeout = timeout

        return await self._unary_unary(
            "/nnext.Collections/Update", request, CollectionOperationResponse
        )

    async def delete(
        self, *, collection_name: str = "", timeout: Optional[int] = None
    ) -> "CollectionOperationResponse":

        request = DeleteCollection()
        request.collection_name = collection_name
        request.timeout = timeout

        return await self._unary_unary(
            "/nnext.Collections/Delete", request, CollectionOperationResponse
        )

    async def update_aliases(
        self,
        *,
        actions: Optional[List["AliasOperations"]] = None,
        timeout: Optional[int] = None
    ) -> "CollectionOperationResponse":
        actions = actions or []

        request = ChangeAliases()
        if actions is not None:
            request.actions = actions
        request.timeout = timeout

        return await self._unary_unary(
            "/nnext.Collections/UpdateAliases", request, CollectionOperationResponse
        )


class PointsStub(betterproto.ServiceStub):
    async def upsert(
        self,
        *,
        collection_name: str = "",
        wait: Optional[bool] = None,
        points: Optional[List["PointStruct"]] = None
    ) -> "PointsOperationResponse":
        points = points or []

        request = UpsertPoints()
        request.collection_name = collection_name
        request.wait = wait
        if points is not None:
            request.points = points

        return await self._unary_unary(
            "/nnext.Points/Upsert", request, PointsOperationResponse
        )

    async def delete(
        self,
        *,
        collection_name: str = "",
        wait: Optional[bool] = None,
        points: "PointsSelector" = None
    ) -> "PointsOperationResponse":

        request = DeletePoints()
        request.collection_name = collection_name
        request.wait = wait
        if points is not None:
            request.points = points

        return await self._unary_unary(
            "/nnext.Points/Delete", request, PointsOperationResponse
        )

    async def get(
        self,
        *,
        collection_name: str = "",
        ids: Optional[List["PointId"]] = None,
        with_vector: Optional[bool] = None,
        with_payload: "WithPayloadSelector" = None
    ) -> "GetResponse":
        ids = ids or []

        request = GetPoints()
        request.collection_name = collection_name
        if ids is not None:
            request.ids = ids
        request.with_vector = with_vector
        if with_payload is not None:
            request.with_payload = with_payload

        return await self._unary_unary("/nnext.Points/Get", request, GetResponse)

    async def set_payload(
        self,
        *,
        collection_name: str = "",
        wait: Optional[bool] = None,
        payload: Dict[str, "Value"] = None,
        points: Optional[List["PointId"]] = None
    ) -> "PointsOperationResponse":
        points = points or []

        request = SetPayloadPoints()
        request.collection_name = collection_name
        request.wait = wait
        request.payload = payload
        if points is not None:
            request.points = points

        return await self._unary_unary(
            "/nnext.Points/SetPayload", request, PointsOperationResponse
        )

    async def delete_payload(
        self,
        *,
        collection_name: str = "",
        wait: Optional[bool] = None,
        keys: Optional[List[str]] = None,
        points: Optional[List["PointId"]] = None
    ) -> "PointsOperationResponse":
        keys = keys or []
        points = points or []

        request = DeletePayloadPoints()
        request.collection_name = collection_name
        request.wait = wait
        request.keys = keys
        if points is not None:
            request.points = points

        return await self._unary_unary(
            "/nnext.Points/DeletePayload", request, PointsOperationResponse
        )

    async def clear_payload(
        self,
        *,
        collection_name: str = "",
        wait: Optional[bool] = None,
        points: "PointsSelector" = None
    ) -> "PointsOperationResponse":

        request = ClearPayloadPoints()
        request.collection_name = collection_name
        request.wait = wait
        if points is not None:
            request.points = points

        return await self._unary_unary(
            "/nnext.Points/ClearPayload", request, PointsOperationResponse
        )

    async def create_field_index(
        self,
        *,
        collection_name: str = "",
        wait: Optional[bool] = None,
        field_name: str = "",
        field_type: Optional["FieldType"] = None
    ) -> "PointsOperationResponse":

        request = CreateFieldIndexCollection()
        request.collection_name = collection_name
        request.wait = wait
        request.field_name = field_name
        request.field_type = field_type

        return await self._unary_unary(
            "/nnext.Points/CreateFieldIndex", request, PointsOperationResponse
        )

    async def delete_field_index(
        self,
        *,
        collection_name: str = "",
        wait: Optional[bool] = None,
        field_name: str = ""
    ) -> "PointsOperationResponse":

        request = DeleteFieldIndexCollection()
        request.collection_name = collection_name
        request.wait = wait
        request.field_name = field_name

        return await self._unary_unary(
            "/nnext.Points/DeleteFieldIndex", request, PointsOperationResponse
        )

    async def search(
        self,
        *,
        collection_name: str = "",
        vector: Optional[List[float]] = None,
        filter: "Filter" = None,
        limit: int = 0,
        with_vector: Optional[bool] = None,
        with_payload: "WithPayloadSelector" = None,
        params: "SearchParams" = None,
        score_threshold: Optional[float] = None,
        offset: Optional[int] = None
    ) -> "SearchResponse":
        vector = vector or []

        request = SearchPoints()
        request.collection_name = collection_name
        request.vector = vector
        if filter is not None:
            request.filter = filter
        request.limit = limit
        request.with_vector = with_vector
        if with_payload is not None:
            request.with_payload = with_payload
        if params is not None:
            request.params = params
        request.score_threshold = score_threshold
        request.offset = offset

        return await self._unary_unary("/nnext.Points/Search", request, SearchResponse)

    async def scroll(
        self,
        *,
        collection_name: str = "",
        filter: "Filter" = None,
        offset: Optional["PointId"] = None,
        limit: Optional[int] = None,
        with_vector: Optional[bool] = None,
        with_payload: "WithPayloadSelector" = None
    ) -> "ScrollResponse":

        request = ScrollPoints()
        request.collection_name = collection_name
        if filter is not None:
            request.filter = filter
        if offset is not None:
            request.offset = offset
        request.limit = limit
        request.with_vector = with_vector
        if with_payload is not None:
            request.with_payload = with_payload

        return await self._unary_unary("/nnext.Points/Scroll", request, ScrollResponse)

    async def recommend(
        self,
        *,
        collection_name: str = "",
        positive: Optional[List["PointId"]] = None,
        negative: Optional[List["PointId"]] = None,
        filter: "Filter" = None,
        limit: int = 0,
        with_vector: Optional[bool] = None,
        with_payload: "WithPayloadSelector" = None,
        params: "SearchParams" = None,
        score_threshold: Optional[float] = None,
        offset: Optional[int] = None
    ) -> "RecommendResponse":
        positive = positive or []
        negative = negative or []

        request = RecommendPoints()
        request.collection_name = collection_name
        if positive is not None:
            request.positive = positive
        if negative is not None:
            request.negative = negative
        if filter is not None:
            request.filter = filter
        request.limit = limit
        request.with_vector = with_vector
        if with_payload is not None:
            request.with_payload = with_payload
        if params is not None:
            request.params = params
        request.score_threshold = score_threshold
        request.offset = offset

        return await self._unary_unary(
            "/nnext.Points/Recommend", request, RecommendResponse
        )

    async def count(
        self,
        *,
        collection_name: str = "",
        filter: "Filter" = None,
        exact: Optional[bool] = None
    ) -> "CountResponse":

        request = CountPoints()
        request.collection_name = collection_name
        if filter is not None:
            request.filter = filter
        request.exact = exact

        return await self._unary_unary("/nnext.Points/Count", request, CountResponse)


class SnapshotsStub(betterproto.ServiceStub):
    async def create(self, *, collection_name: str = "") -> "CreateSnapshotResponse":

        request = CreateSnapshotRequest()
        request.collection_name = collection_name

        return await self._unary_unary(
            "/nnext.Snapshots/Create", request, CreateSnapshotResponse
        )

    async def list(self, *, collection_name: str = "") -> "ListSnapshotsResponse":

        request = ListSnapshotsRequest()
        request.collection_name = collection_name

        return await self._unary_unary(
            "/nnext.Snapshots/List", request, ListSnapshotsResponse
        )

    async def create_full(self) -> "CreateSnapshotResponse":

        request = CreateFullSnapshotRequest()

        return await self._unary_unary(
            "/nnext.Snapshots/CreateFull", request, CreateSnapshotResponse
        )

    async def list_full(self) -> "ListSnapshotsResponse":

        request = ListFullSnapshotsRequest()

        return await self._unary_unary(
            "/nnext.Snapshots/ListFull", request, ListSnapshotsResponse
        )


class NNextStub(betterproto.ServiceStub):
    async def health_check(self) -> "HealthCheckReply":

        request = HealthCheckRequest()

        return await self._unary_unary(
            "/nnext.NNext/HealthCheck", request, HealthCheckReply
        )


class CollectionsBase(ServiceBase):
    async def get(self, collection_name: str) -> "GetCollectionInfoResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list(self) -> "ListCollectionsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create(
        self,
        collection_name: str,
        vector_size: int,
        distance: "Distance",
        hnsw_config: Optional["HnswConfigDiff"],
        wal_config: Optional["WalConfigDiff"],
        optimizers_config: Optional["OptimizersConfigDiff"],
        shard_number: Optional[int],
        on_disk_payload: Optional[bool],
        timeout: Optional[int],
    ) -> "CollectionOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update(
        self,
        collection_name: str,
        optimizers_config: Optional["OptimizersConfigDiff"],
        timeout: Optional[int],
    ) -> "CollectionOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(
        self, collection_name: str, timeout: Optional[int]
    ) -> "CollectionOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_aliases(
        self, actions: Optional[List["AliasOperations"]], timeout: Optional[int]
    ) -> "CollectionOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
        }

        response = await self.get(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_create(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "vector_size": request.vector_size,
            "distance": request.distance,
            "hnsw_config": request.hnsw_config,
            "wal_config": request.wal_config,
            "optimizers_config": request.optimizers_config,
            "shard_number": request.shard_number,
            "on_disk_payload": request.on_disk_payload,
            "timeout": request.timeout,
        }

        response = await self.create(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_update(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "optimizers_config": request.optimizers_config,
            "timeout": request.timeout,
        }

        response = await self.update(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "timeout": request.timeout,
        }

        response = await self.delete(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_update_aliases(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "actions": request.actions,
            "timeout": request.timeout,
        }

        response = await self.update_aliases(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nnext.Collections/Get": grpclib.const.Handler(
                self.__rpc_get,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetCollectionInfoRequest,
                GetCollectionInfoResponse,
            ),
            "/nnext.Collections/List": grpclib.const.Handler(
                self.__rpc_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListCollectionsRequest,
                ListCollectionsResponse,
            ),
            "/nnext.Collections/Create": grpclib.const.Handler(
                self.__rpc_create,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateCollection,
                CollectionOperationResponse,
            ),
            "/nnext.Collections/Update": grpclib.const.Handler(
                self.__rpc_update,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateCollection,
                CollectionOperationResponse,
            ),
            "/nnext.Collections/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteCollection,
                CollectionOperationResponse,
            ),
            "/nnext.Collections/UpdateAliases": grpclib.const.Handler(
                self.__rpc_update_aliases,
                grpclib.const.Cardinality.UNARY_UNARY,
                ChangeAliases,
                CollectionOperationResponse,
            ),
        }


class PointsBase(ServiceBase):
    async def upsert(
        self,
        collection_name: str,
        wait: Optional[bool],
        points: Optional[List["PointStruct"]],
    ) -> "PointsOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(
        self, collection_name: str, wait: Optional[bool], points: "PointsSelector"
    ) -> "PointsOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get(
        self,
        collection_name: str,
        ids: Optional[List["PointId"]],
        with_vector: Optional[bool],
        with_payload: "WithPayloadSelector",
    ) -> "GetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_payload(
        self,
        collection_name: str,
        wait: Optional[bool],
        payload: Dict[str, "Value"],
        points: Optional[List["PointId"]],
    ) -> "PointsOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_payload(
        self,
        collection_name: str,
        wait: Optional[bool],
        keys: Optional[List[str]],
        points: Optional[List["PointId"]],
    ) -> "PointsOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def clear_payload(
        self, collection_name: str, wait: Optional[bool], points: "PointsSelector"
    ) -> "PointsOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create_field_index(
        self,
        collection_name: str,
        wait: Optional[bool],
        field_name: str,
        field_type: Optional["FieldType"],
    ) -> "PointsOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_field_index(
        self, collection_name: str, wait: Optional[bool], field_name: str
    ) -> "PointsOperationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def search(
        self,
        collection_name: str,
        vector: Optional[List[float]],
        filter: "Filter",
        limit: int,
        with_vector: Optional[bool],
        with_payload: "WithPayloadSelector",
        params: "SearchParams",
        score_threshold: Optional[float],
        offset: Optional[int],
    ) -> "SearchResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def scroll(
        self,
        collection_name: str,
        filter: "Filter",
        offset: Optional["PointId"],
        limit: Optional[int],
        with_vector: Optional[bool],
        with_payload: "WithPayloadSelector",
    ) -> "ScrollResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def recommend(
        self,
        collection_name: str,
        positive: Optional[List["PointId"]],
        negative: Optional[List["PointId"]],
        filter: "Filter",
        limit: int,
        with_vector: Optional[bool],
        with_payload: "WithPayloadSelector",
        params: "SearchParams",
        score_threshold: Optional[float],
        offset: Optional[int],
    ) -> "RecommendResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def count(
        self, collection_name: str, filter: "Filter", exact: Optional[bool]
    ) -> "CountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_upsert(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "wait": request.wait,
            "points": request.points,
        }

        response = await self.upsert(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "wait": request.wait,
            "points": request.points,
        }

        response = await self.delete(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "ids": request.ids,
            "with_vector": request.with_vector,
            "with_payload": request.with_payload,
        }

        response = await self.get(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set_payload(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "wait": request.wait,
            "payload": request.payload,
            "points": request.points,
        }

        response = await self.set_payload(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_payload(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "wait": request.wait,
            "keys": request.keys,
            "points": request.points,
        }

        response = await self.delete_payload(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_clear_payload(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "wait": request.wait,
            "points": request.points,
        }

        response = await self.clear_payload(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_create_field_index(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "wait": request.wait,
            "field_name": request.field_name,
            "field_type": request.field_type,
        }

        response = await self.create_field_index(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_field_index(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "wait": request.wait,
            "field_name": request.field_name,
        }

        response = await self.delete_field_index(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_search(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "vector": request.vector,
            "filter": request.filter,
            "limit": request.limit,
            "with_vector": request.with_vector,
            "with_payload": request.with_payload,
            "params": request.params,
            "score_threshold": request.score_threshold,
            "offset": request.offset,
        }

        response = await self.search(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_scroll(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "filter": request.filter,
            "offset": request.offset,
            "limit": request.limit,
            "with_vector": request.with_vector,
            "with_payload": request.with_payload,
        }

        response = await self.scroll(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_recommend(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "positive": request.positive,
            "negative": request.negative,
            "filter": request.filter,
            "limit": request.limit,
            "with_vector": request.with_vector,
            "with_payload": request.with_payload,
            "params": request.params,
            "score_threshold": request.score_threshold,
            "offset": request.offset,
        }

        response = await self.recommend(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_count(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
            "filter": request.filter,
            "exact": request.exact,
        }

        response = await self.count(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nnext.Points/Upsert": grpclib.const.Handler(
                self.__rpc_upsert,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpsertPoints,
                PointsOperationResponse,
            ),
            "/nnext.Points/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeletePoints,
                PointsOperationResponse,
            ),
            "/nnext.Points/Get": grpclib.const.Handler(
                self.__rpc_get,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetPoints,
                GetResponse,
            ),
            "/nnext.Points/SetPayload": grpclib.const.Handler(
                self.__rpc_set_payload,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetPayloadPoints,
                PointsOperationResponse,
            ),
            "/nnext.Points/DeletePayload": grpclib.const.Handler(
                self.__rpc_delete_payload,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeletePayloadPoints,
                PointsOperationResponse,
            ),
            "/nnext.Points/ClearPayload": grpclib.const.Handler(
                self.__rpc_clear_payload,
                grpclib.const.Cardinality.UNARY_UNARY,
                ClearPayloadPoints,
                PointsOperationResponse,
            ),
            "/nnext.Points/CreateFieldIndex": grpclib.const.Handler(
                self.__rpc_create_field_index,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateFieldIndexCollection,
                PointsOperationResponse,
            ),
            "/nnext.Points/DeleteFieldIndex": grpclib.const.Handler(
                self.__rpc_delete_field_index,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteFieldIndexCollection,
                PointsOperationResponse,
            ),
            "/nnext.Points/Search": grpclib.const.Handler(
                self.__rpc_search,
                grpclib.const.Cardinality.UNARY_UNARY,
                SearchPoints,
                SearchResponse,
            ),
            "/nnext.Points/Scroll": grpclib.const.Handler(
                self.__rpc_scroll,
                grpclib.const.Cardinality.UNARY_UNARY,
                ScrollPoints,
                ScrollResponse,
            ),
            "/nnext.Points/Recommend": grpclib.const.Handler(
                self.__rpc_recommend,
                grpclib.const.Cardinality.UNARY_UNARY,
                RecommendPoints,
                RecommendResponse,
            ),
            "/nnext.Points/Count": grpclib.const.Handler(
                self.__rpc_count,
                grpclib.const.Cardinality.UNARY_UNARY,
                CountPoints,
                CountResponse,
            ),
        }


class SnapshotsBase(ServiceBase):
    async def create(self, collection_name: str) -> "CreateSnapshotResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list(self, collection_name: str) -> "ListSnapshotsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create_full(self) -> "CreateSnapshotResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_full(self) -> "ListSnapshotsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
        }

        response = await self.create(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection_name": request.collection_name,
        }

        response = await self.list(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_create_full(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.create_full(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_full(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list_full(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nnext.Snapshots/Create": grpclib.const.Handler(
                self.__rpc_create,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateSnapshotRequest,
                CreateSnapshotResponse,
            ),
            "/nnext.Snapshots/List": grpclib.const.Handler(
                self.__rpc_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListSnapshotsRequest,
                ListSnapshotsResponse,
            ),
            "/nnext.Snapshots/CreateFull": grpclib.const.Handler(
                self.__rpc_create_full,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateFullSnapshotRequest,
                CreateSnapshotResponse,
            ),
            "/nnext.Snapshots/ListFull": grpclib.const.Handler(
                self.__rpc_list_full,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListFullSnapshotsRequest,
                ListSnapshotsResponse,
            ),
        }


class NNextBase(ServiceBase):
    async def health_check(self) -> "HealthCheckReply":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_health_check(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.health_check(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nnext.NNext/HealthCheck": grpclib.const.Handler(
                self.__rpc_health_check,
                grpclib.const.Cardinality.UNARY_UNARY,
                HealthCheckRequest,
                HealthCheckReply,
            ),
        }



from datetime import datetime
from typing import Dict, Any

import betterproto

from nnext import grpc
from nnext.http.models import models as rest
from nnext.grpc import Value, ListValue, Struct, NullValue


def json_to_value(payload: Any) -> Value:
    if payload is None:
        return Value(null_value=NullValue.NULL_VALUE)
    if isinstance(payload, bool):
        return Value(bool_value=payload)
    if isinstance(payload, int):
        return Value(integer_value=payload)
    if isinstance(payload, float):
        return Value(double_value=payload)
    if isinstance(payload, str):
        return Value(string_value=payload)
    if isinstance(payload, list):
        return Value(list_value=ListValue(values=[json_to_value(v) for v in payload]))
    if isinstance(payload, dict):
        return Value(struct_value=Struct(fields=dict((k, json_to_value(v)) for k, v in payload.items())))
    raise ValueError(f"Not supported json value: {payload}")  # pragma: no cover


def value_to_json(value: Value) -> Any:
    if isinstance(value, Value):
        value_ = value.to_dict(casing=betterproto.Casing.CAMEL)
    else:
        value_ = value

    if "integerValue" in value_:
        # by default int are represented as string for precision
        # But in python it is OK to just use `int`
        return int(value_["integerValue"])
    if "doubleValue" in value_:
        return value_["doubleValue"]
    if "stringValue" in value_:
        return value_["stringValue"]
    if "boolValue" in value_:
        return value_["boolValue"]
    if "structValue" in value_:
        if 'fields' not in value_['structValue']:
            return {}
        return dict((key, value_to_json(val)) for key, val in value_["structValue"]['fields'].items())
    if "listValue" in value_:
        if 'values' in value_["listValue"]:
            return list(value_to_json(val) for val in value_["listValue"]['values'])
        else:
            return []
    if "nullValue" in value_:
        return None
    raise ValueError(f"Not supported value: {value_}")  # pragma: no cover


def payload_to_grpc(payload: Dict[str, Any]) -> Dict[str, Value]:
    return dict(
        (key, json_to_value(val))
        for key, val in payload.items()
    )


def grpc_to_payload(grpc: Dict[str, Value]) -> Dict[str, Any]:
    return dict(
        (key, value_to_json(val))
        for key, val in grpc.items()
    )


class GrpcToRest:

    @classmethod
    def convert_condition(cls, model: grpc.Condition) -> rest.Condition:
        name, val = betterproto.which_one_of(model, "condition_one_of")

        if name == "field":
            return cls.convert_field_condition(val)
        if name == "filter":
            return cls.convert_filter(val)
        if name == "has_id":
            return cls.convert_has_id_condition(val)
        if name == "is_empty":
            return cls.convert_is_empty_condition(val)

        raise ValueError(f"invalid Condition model: {model}")  # pragma: no cover

    @classmethod
    def convert_filter(cls, model: grpc.Filter) -> rest.Filter:
        return rest.Filter(
            must=[cls.convert_condition(condition) for condition in model.must],
            should=[cls.convert_condition(condition) for condition in model.should],
            must_not=[cls.convert_condition(condition) for condition in model.must_not]
        )

    @classmethod
    def convert_range(cls, model: grpc.Range) -> rest.Range:
        return rest.Range(
            gt=model.gt,
            gte=model.gte,
            lt=model.lt,
            lte=model.lte,
        )

    @classmethod
    def convert_geo_radius(cls, model: grpc.GeoRadius) -> rest.GeoRadius:
        return rest.GeoRadius(
            center=cls.convert_geo_point(model.center),
            radius=model.radius
        )

    @classmethod
    def convert_collection_description(cls, model: grpc.CollectionDescription) -> rest.CollectionDescription:
        return rest.CollectionDescription(name=model.name)

    @classmethod
    def convert_collection_info(cls, model: grpc.CollectionInfo) -> rest.CollectionInfo:
        return rest.CollectionInfo(
            config=cls.convert_collection_config(model.config),
            disk_data_size=model.disk_data_size,
            optimizer_status=cls.convert_optimizer_status(model.optimizer_status),
            payload_schema=cls.convert_payload_schema(model.payload_schema),
            ram_data_size=model.ram_data_size,
            segments_count=model.segments_count,
            status=cls.convert_collection_status(model.status),
            vectors_count=model.vectors_count,
            points_count=model.points_count,
            indexed_vectors_count=model.indexed_vectors_count or 0,
        )

    @classmethod
    def convert_optimizer_status(cls, model: grpc.OptimizerStatus) -> rest.OptimizersStatus:
        if model.ok:
            return rest.OptimizersStatusOneOf.OK
        else:
            return rest.OptimizersStatusOneOf1(error=model.error)

    @classmethod
    def convert_collection_config(cls, model: grpc.CollectionConfig) -> rest.CollectionConfig:
        return rest.CollectionConfig(
            hnsw_config=cls.convert_hnsw_config_diff(model.hnsw_config),
            optimizer_config=cls.convert_optimizer_config(model.optimizer_config),
            params=cls.convert_collection_params(model.params),
            wal_config=cls.convert_wal_config(model.wal_config)
        )

    @classmethod
    def convert_hnsw_config_diff(cls, model: grpc.HnswConfigDiff) -> rest.HnswConfigDiff:
        return rest.HnswConfigDiff(
            ef_construct=model.ef_construct,
            m=model.m,
            full_scan_threshold=model.full_scan_threshold
        )

    @classmethod
    def convert_hnsw_config(cls, model: grpc.HnswConfigDiff) -> rest.HnswConfig:
        return rest.HnswConfig(
            ef_construct=model.ef_construct,
            m=model.m,
            full_scan_threshold=model.full_scan_threshold
        )

    @classmethod
    def convert_optimizer_config(cls, model: grpc.OptimizersConfigDiff) -> rest.OptimizersConfig:
        return rest.OptimizersConfig(
            default_segment_number=model.default_segment_number,
            deleted_threshold=model.deleted_threshold,
            flush_interval_sec=model.flush_interval_sec,
            indexing_threshold=model.indexing_threshold,
            max_optimization_threads=model.max_optimization_threads,
            max_segment_size=model.max_segment_size,
            memmap_threshold=model.memmap_threshold,
            vacuum_min_vector_number=model.vacuum_min_vector_number
        )

    @classmethod
    def convert_distance(cls, model: grpc.Distance) -> rest.Distance:
        if model == grpc.Distance.Cosine:
            return rest.Distance.COSINE
        elif model == grpc.Distance.Euclid:
            return rest.Distance.EUCLID
        elif model == grpc.Distance.Dot:
            return rest.Distance.DOT
        else:
            raise ValueError(f"invalid Distance model: {model}")  # pragma: no cover

    @classmethod
    def convert_wal_config(cls, model: grpc.WalConfigDiff) -> rest.WalConfig:
        return rest.WalConfig(wal_capacity_mb=model.wal_capacity_mb,
                              wal_segments_ahead=model.wal_segments_ahead)

    @classmethod
    def convert_payload_schema(cls, model: Dict[str, grpc.PayloadSchemaInfo]) -> Dict[str, rest.PayloadIndexInfo]:
        return {key: cls.convert_payload_schema_info(info) for key, info in model.items()}

    @classmethod
    def convert_payload_schema_info(cls, model: grpc.PayloadSchemaInfo) -> rest.PayloadIndexInfo:
        return rest.PayloadIndexInfo(data_type=cls.convert_payload_schema_type(model.data_type))

    @classmethod
    def convert_payload_schema_type(cls, model: grpc.PayloadSchemaType) -> rest.PayloadSchemaType:
        if model == grpc.PayloadSchemaType.Float:
            return rest.PayloadSchemaType.FLOAT
        elif model == grpc.PayloadSchemaType.Geo:
            return rest.PayloadSchemaType.GEO
        elif model == grpc.PayloadSchemaType.Integer:
            return rest.PayloadSchemaType.INTEGER
        elif model == grpc.PayloadSchemaType.Keyword:
            return rest.PayloadSchemaType.KEYWORD
        else:
            raise ValueError(f"invalid PayloadSchemaType model: {model}")  # pragma: no cover

    @classmethod
    def convert_collection_status(cls, model: grpc.CollectionStatus) -> rest.CollectionStatus:
        if model == grpc.CollectionStatus.Green:
            return rest.CollectionStatus.GREEN
        elif model == grpc.CollectionStatus.Yellow:
            return rest.CollectionStatus.YELLOW
        elif model == grpc.CollectionStatus.Red:
            return rest.CollectionStatus.RED
        else:
            raise ValueError(f"invalid CollectionStatus model: {model}")  # pragma: no cover

    @classmethod
    def convert_update_result(cls, model: grpc.UpdateResult) -> rest.UpdateResult:
        return rest.UpdateResult(operation_id=model.operation_id, status=cls.convert_update_status(model.status))

    @classmethod
    def convert_update_status(cls, model: grpc.UpdateStatus) -> rest.UpdateStatus:
        if model == grpc.UpdateStatus.Acknowledged:
            return rest.UpdateStatus.ACKNOWLEDGED
        elif model == grpc.UpdateStatus.Completed:
            return rest.UpdateStatus.COMPLETED
        else:
            raise ValueError(f"invalid UpdateStatus model: {model}")  # pragma: no cover

    @classmethod
    def convert_has_id_condition(cls, model: grpc.HasIdCondition) -> rest.HasIdCondition:
        return rest.HasIdCondition(
            has_id=[cls.convert_point_id(idx) for idx in model.has_id]
        )

    @classmethod
    def convert_point_id(cls, model: grpc.PointId) -> rest.ExtendedPointId:
        name, val = betterproto.which_one_of(model, "point_id_options")

        if name == "num":
            return val
        if name == "uuid":
            return val
        raise ValueError(f"invalid PointId model: {model}")  # pragma: no cover

    @classmethod
    def convert_delete_alias(cls, model: grpc.DeleteAlias) -> rest.DeleteAlias:
        return rest.DeleteAlias(alias_name=model.alias_name)

    @classmethod
    def convert_rename_alias(cls, model: grpc.RenameAlias) -> rest.RenameAlias:
        return rest.RenameAlias(old_alias_name=model.old_alias_name, new_alias_name=model.new_alias_name)

    @classmethod
    def convert_is_empty_condition(cls, model: grpc.IsEmptyCondition) -> rest.IsEmptyCondition:
        return rest.IsEmptyCondition(is_empty=rest.PayloadField(key=model.key))

    @classmethod
    def convert_search_params(cls, model: grpc.SearchParams) -> rest.SearchParams:
        return rest.SearchParams(hnsw_ef=model.hnsw_ef)

    @classmethod
    def convert_create_alias(cls, model: grpc.CreateAlias) -> rest.CreateAlias:
        return rest.CreateAlias(
            collection_name=model.collection_name,
            alias_name=model.alias_name
        )

    @classmethod
    def convert_create_collection(cls, model: grpc.CreateCollection) -> rest.CreateCollection:
        return rest.CreateCollection(
            collection_name=model.collection_name,
            vector_size=model.vector_size,
            distance=cls.convert_distance(model.distance),
            hnsw_config=cls.convert_hnsw_config(model.hnsw_config),
            wal_config=cls.convert_wal_config(model.wal_config),
            optimizers_config=cls.convert_optimizer_config(model.optimizers_config),
            shard_number=model.shard_number
        )

    @classmethod
    def convert_scored_point(cls, model: grpc.ScoredPoint) -> rest.ScoredPoint:
        return rest.ScoredPoint(
            id=cls.convert_point_id(model.id),
            payload=cls.convert_payload(model.payload) if model.payload is not None else None,
            score=model.score,
            vector=model.vector,
            version=model.version,
        )

    @classmethod
    def convert_payload(cls, model: Dict[str, grpc.Value]) -> rest.Payload:
        return dict(
            (key, value_to_json(val))
            for key, val in
            model.items()
        )

    @classmethod
    def convert_values_count(cls, model: grpc.ValuesCount) -> rest.ValuesCount:
        return rest.ValuesCount(
            gt=model.gt,
            gte=model.gte,
            lt=model.lt,
            lte=model.lte,
        )

    @classmethod
    def convert_geo_bounding_box(cls, model: grpc.GeoBoundingBox) -> rest.GeoBoundingBox:
        return rest.GeoBoundingBox(
            bottom_right=cls.convert_geo_point(model.bottom_right),
            top_left=cls.convert_geo_point(model.top_left)
        )

    @classmethod
    def convert_point_struct(cls, model: grpc.PointStruct) -> rest.PointStruct:
        return rest.PointStruct(
            id=cls.convert_point_id(model.id),
            payload=cls.convert_payload(model.payload),
            vector=model.vector,
        )

    @classmethod
    def convert_field_condition(cls, model: grpc.FieldCondition) -> rest.FieldCondition:
        geo_bounding_box = cls.convert_geo_bounding_box(model.geo_bounding_box) if model.geo_bounding_box else None
        geo_radius = cls.convert_geo_radius(model.geo_radius) if model.geo_radius else None
        match = cls.convert_match(model.match) if model.match else None
        range_ = cls.convert_range(model.range) if model.range else None
        values_count = cls.convert_values_count(model.values_count) if model.values_count else None
        return rest.FieldCondition(
            key=model.key,
            geo_bounding_box=geo_bounding_box,
            geo_radius=geo_radius,
            match=match,
            range=range_,
            values_count=values_count,
        )

    @classmethod
    def convert_match(cls, model: grpc.Match) -> rest.Match:
        name, val = betterproto.which_one_of(model, "match_value")

        if name == "integer":
            return rest.MatchValue(value=val)
        if name == "boolean":
            return rest.MatchValue(value=val)
        if name == "keyword":
            return rest.MatchValue(value=val)
        raise ValueError(f"invalid Match model: {model}")  # pragma: no cover

    @classmethod
    def convert_wal_config_diff(cls, model: grpc.WalConfigDiff) -> rest.WalConfigDiff:
        return rest.WalConfigDiff(
            wal_capacity_mb=model.wal_capacity_mb,
            wal_segments_ahead=model.wal_segments_ahead
        )

    @classmethod
    def convert_collection_params(cls, model: grpc.CollectionParams) -> rest.CollectionParams:
        return rest.CollectionParams(
            distance=cls.convert_distance(model.distance),
            shard_number=model.shard_number,
            vector_size=model.vector_size
        )

    @classmethod
    def convert_optimizers_config_diff(cls, model: grpc.OptimizersConfigDiff) -> rest.OptimizersConfigDiff:
        return rest.OptimizersConfigDiff(
            default_segment_number=model.default_segment_number,
            deleted_threshold=model.deleted_threshold,
            flush_interval_sec=model.flush_interval_sec,
            indexing_threshold=model.indexing_threshold,
            max_optimization_threads=model.max_optimization_threads,
            max_segment_size=model.max_segment_size,
            memmap_threshold=model.memmap_threshold,
            vacuum_min_vector_number=model.vacuum_min_vector_number,
        )

    @classmethod
    def convert_update_collection(cls, model: grpc.UpdateCollection) -> rest.UpdateCollection:
        return rest.UpdateCollection(
            optimizers_config=cls.convert_optimizers_config_diff(
                model.optimizers_config) if model.optimizers_config is not None else None
        )

    @classmethod
    def convert_geo_point(cls, model: grpc.GeoPoint) -> rest.GeoPoint:
        return rest.GeoPoint(
            lat=model.lat,
            lon=model.lon,
        )

    @classmethod
    def convert_alias_operations(cls, model: grpc.AliasOperations) -> rest.AliasOperations:
        name, val = betterproto.which_one_of(model, "action")

        if name == "rename_alias":
            return rest.RenameAliasOperation(rename_alias=cls.convert_rename_alias(val))
        if name == "create_alias":
            return rest.CreateAliasOperation(create_alias=cls.convert_create_alias(val))
        if name == "delete_alias":
            return rest.DeleteAliasOperation(delete_alias=cls.convert_delete_alias(val))

        raise ValueError(f"invalid AliasOperations model: {model}")  # pragma: no cover

    @classmethod
    def convert_points_selector(cls, model: grpc.PointsSelector) -> rest.PointsSelector:
        name, val = betterproto.which_one_of(model, "points_selector_one_of")

        if name == "points":
            return rest.PointIdsList(points=[
                cls.convert_point_id(point)
                for point in val.ids
            ])
        if name == "filter":
            return rest.FilterSelector(
                filter=cls.convert_filter(val)
            )
        raise ValueError(f"invalid PointsSelector model: {model}")  # pragma: no cover

    @classmethod
    def convert_with_payload_selector(cls, model: grpc.WithPayloadSelector) -> rest.WithPayloadInterface:
        name, val = betterproto.which_one_of(model, "selector_options")

        if name == "enable":
            return val
        if name == "include":
            return val.fields
        if name == "exclude":
            return rest.PayloadSelectorExclude(exclude=val.fields)

        raise ValueError(f"invalid WithPayloadSelector model: {model}")  # pragma: no cover

    @classmethod
    def convert_with_payload_interface(cls, model: grpc.WithPayloadSelector) -> rest.WithPayloadInterface:
        return cls.convert_with_payload_selector(model)

    @classmethod
    def convert_retrieved_point(cls, model: grpc.RetrievedPoint) -> rest.Record:
        return rest.Record(
            id=cls.convert_point_id(model.id),
            payload=cls.convert_payload(model.payload) if model.payload is not None else None,
            vector=model.vector
        )

    @classmethod
    def convert_record(cls, model: grpc.RetrievedPoint) -> rest.Record:
        return cls.convert_retrieved_point(model)

    @classmethod
    def convert_count_result(cls, model: grpc.CountResult) -> rest.CountResult:
        return rest.CountResult(
            count=model.count
        )

    @classmethod
    def convert_snapshot_description(cls, model: grpc.SnapshotDescription) -> rest.SnapshotDescription:
        return rest.SnapshotDescription(
            name=model.name,
            creation_time=model.creation_time.isoformat(),
            size=model.size,
        )


# ----------------------------------------
#
# ----------- REST TO gRPC ---------------
#
# ----------------------------------------

class RestToGrpc:
    @classmethod
    def convert_filter(cls, model: rest.Filter) -> grpc.Filter:
        return grpc.Filter(
            must=[cls.convert_condition(condition) for condition in model.must] if model.must is not None else None,
            must_not=[cls.convert_condition(condition) for condition in
                      model.must_not] if model.must_not is not None else None,
            should=[cls.convert_condition(condition) for condition in
                    model.should] if model.should is not None else None,
        )

    @classmethod
    def convert_range(cls, model: rest.Range) -> grpc.Range:
        return grpc.Range(
            lt=model.lt,
            gt=model.gt,
            gte=model.gte,
            lte=model.lte,
        )

    @classmethod
    def convert_geo_radius(cls, model: rest.GeoRadius) -> grpc.GeoRadius:
        return grpc.GeoRadius(
            center=cls.convert_geo_point(model.center),
            radius=model.radius
        )

    @classmethod
    def convert_collection_description(cls, model: rest.CollectionDescription) -> grpc.CollectionDescription:
        return grpc.CollectionDescription(
            name=model.name
        )

    @classmethod
    def convert_collection_info(cls, model: rest.CollectionInfo) -> grpc.CollectionInfo:
        return grpc.CollectionInfo(
            config=cls.convert_collection_config(model.config) if model.config else None,
            disk_data_size=model.disk_data_size,
            optimizer_status=cls.convert_optimizer_status(model.optimizer_status),
            payload_schema=cls.convert_payload_schema(
                model.payload_schema) if model.payload_schema is not None else None,
            ram_data_size=model.ram_data_size,
            segments_count=model.segments_count,
            status=cls.convert_collection_status(model.status),
            vectors_count=model.vectors_count,
            points_count=model.points_count,
        )

    @classmethod
    def convert_collection_status(cls, model: rest.CollectionStatus) -> grpc.CollectionStatus:
        if model == rest.CollectionStatus.RED:
            return grpc.CollectionStatus.Red
        if model == rest.CollectionStatus.YELLOW:
            return grpc.CollectionStatus.Yellow
        if model == rest.CollectionStatus.GREEN:
            return grpc.CollectionStatus.Green

        raise ValueError(f"invalid CollectionStatus model: {model}")  # pragma: no cover

    @classmethod
    def convert_optimizer_status(cls, model: rest.OptimizersStatus) -> grpc.OptimizerStatus:
        if isinstance(model, rest.OptimizersStatusOneOf):
            return grpc.OptimizerStatus(
                ok=True,
            )
        if isinstance(model, rest.OptimizersStatusOneOf1):
            return grpc.OptimizerStatus(
                ok=False,
                error=model.error
            )
        raise ValueError(f"invalid OptimizersStatus model: {model}")  # pragma: no cover

    @classmethod
    def convert_payload_schema(cls, model: Dict[str, rest.PayloadIndexInfo]) -> Dict[str, grpc.PayloadSchemaInfo]:
        return dict(
            (key, cls.convert_payload_index_info(val))
            for key, val in model.items()
        )

    @classmethod
    def convert_payload_index_info(cls, model: rest.PayloadIndexInfo) -> grpc.PayloadSchemaInfo:
        return grpc.PayloadSchemaInfo(
            data_type=cls.convert_payload_schema_type(model.data_type)
        )

    @classmethod
    def convert_payload_schema_type(cls, model: rest.PayloadSchemaType) -> grpc.PayloadSchemaType:
        if model == rest.PayloadSchemaType.KEYWORD:
            return grpc.PayloadSchemaType.Keyword
        if model == rest.PayloadSchemaType.INTEGER:
            return grpc.PayloadSchemaType.Integer
        if model == rest.PayloadSchemaType.FLOAT:
            return grpc.PayloadSchemaType.Float
        if model == rest.PayloadSchemaType.GEO:
            return grpc.PayloadSchemaType.Geo

        raise ValueError(f"invalid PayloadSchemaType model: {model}")  # pragma: no cover

    @classmethod
    def convert_update_result(cls, model: rest.UpdateResult) -> grpc.UpdateResult:
        return grpc.UpdateResult(
            operation_id=model.operation_id,
            status=cls.convert_update_stats(model.status)
        )

    @classmethod
    def convert_update_stats(cls, model: rest.UpdateStatus) -> grpc.UpdateStatus:
        if model == rest.UpdateStatus.COMPLETED:
            return grpc.UpdateStatus.Completed
        if model == rest.UpdateStatus.ACKNOWLEDGED:
            return grpc.UpdateStatus.Acknowledged

        raise ValueError(f"invalid UpdateStatus model: {model}")  # pragma: no cover

    @classmethod
    def convert_has_id_condition(cls, model: rest.HasIdCondition) -> grpc.HasIdCondition:
        return grpc.HasIdCondition(
            has_id=[cls.convert_extended_point_id(idx) for idx in model.has_id]
        )

    @classmethod
    def convert_delete_alias(cls, model: rest.DeleteAlias) -> grpc.DeleteAlias:
        return grpc.DeleteAlias(
            alias_name=model.alias_name
        )

    @classmethod
    def convert_rename_alias(cls, model: rest.RenameAlias) -> grpc.RenameAlias:
        return grpc.RenameAlias(
            old_alias_name=model.old_alias_name,
            new_alias_name=model.new_alias_name
        )

    @classmethod
    def convert_is_empty_condition(cls, model: rest.IsEmptyCondition) -> grpc.IsEmptyCondition:
        return grpc.IsEmptyCondition(
            key=model.is_empty.key
        )

    @classmethod
    def convert_search_params(cls, model: rest.SearchParams) -> grpc.SearchParams:
        return grpc.SearchParams(
            hnsw_ef=model.hnsw_ef
        )

    @classmethod
    def convert_create_alias(cls, model: rest.CreateAlias) -> grpc.CreateAlias:
        return grpc.CreateAlias(
            collection_name=model.collection_name,
            alias_name=model.alias_name
        )

    @classmethod
    def convert_create_collection(cls, model: rest.CreateCollection, collection_name: str) -> grpc.CreateCollection:
        return grpc.CreateCollection(
            collection_name=collection_name,
            distance=cls.convert_distance(model.distance),
            hnsw_config=cls.convert_hnsw_config_diff(model.hnsw_config) if model.hnsw_config is not None else None,
            optimizers_config=cls.convert_optimizers_config_diff(
                model.optimizers_config) if model.optimizers_config is not None else None,
            shard_number=model.shard_number,
            vector_size=model.vector_size,
            wal_config=cls.convert_wal_config_diff(model.wal_config) if model.wal_config is not None else None,
        )

    @classmethod
    def convert_scored_point(cls, model: rest.ScoredPoint) -> grpc.ScoredPoint:
        return grpc.ScoredPoint(
            id=cls.convert_extended_point_id(model.id),
            payload=cls.convert_payload(model.payload) if model.payload is not None else None,
            score=model.score,
            vector=model.vector,
            version=model.version
        )

    @classmethod
    def convert_values_count(cls, model: rest.ValuesCount) -> grpc.ValuesCount:
        return grpc.ValuesCount(
            lt=model.lt,
            gt=model.gt,
            gte=model.gte,
            lte=model.lte,
        )

    @classmethod
    def convert_geo_bounding_box(cls, model: rest.GeoBoundingBox) -> grpc.GeoBoundingBox:
        return grpc.GeoBoundingBox(
            top_left=cls.convert_geo_point(model.top_left),
            bottom_right=cls.convert_geo_point(model.bottom_right),
        )

    @classmethod
    def convert_point_struct(cls, model: rest.PointStruct) -> grpc.PointStruct:
        return grpc.PointStruct(
            id=cls.convert_extended_point_id(model.id),
            vector=model.vector,
            payload=cls.convert_payload(model.payload) if model.payload is not None else None
        )

    @classmethod
    def convert_payload(cls, model: rest.Payload) -> Dict[str, grpc.Value]:
        return dict((key, json_to_value(val)) for key, val in model.items())

    @classmethod
    def convert_hnsw_config_diff(cls, model: rest.HnswConfigDiff) -> grpc.HnswConfigDiff:
        return grpc.HnswConfigDiff(
            ef_construct=model.ef_construct,
            full_scan_threshold=model.full_scan_threshold,
            m=model.m,
        )

    @classmethod
    def convert_field_condition(cls, model: rest.FieldCondition) -> grpc.FieldCondition:
        if model.match:
            return grpc.FieldCondition(
                key=model.key,
                match=cls.convert_match(model.match)
            )
        if model.range:
            return grpc.FieldCondition(
                key=model.key,
                range=cls.convert_range(model.range)
            )
        if model.geo_bounding_box:
            return grpc.FieldCondition(
                key=model.key,
                geo_bounding_box=cls.convert_geo_bounding_box(model.geo_bounding_box)
            )
        if model.geo_radius:
            return grpc.FieldCondition(
                key=model.key,
                geo_radius=cls.convert_geo_radius(model.geo_radius)
            )
        if model.values_count:
            return grpc.FieldCondition(
                key=model.key,
                values_count=cls.convert_values_count(model.values_count)
            )
        raise ValueError(f"invalid FieldCondition model: {model}")  # pragma: no cover

    @classmethod
    def convert_wal_config_diff(cls, model: rest.WalConfigDiff) -> grpc.WalConfigDiff:
        return grpc.WalConfigDiff(
            wal_capacity_mb=model.wal_capacity_mb,
            wal_segments_ahead=model.wal_segments_ahead
        )

    @classmethod
    def convert_collection_config(cls, model: rest.CollectionConfig) -> grpc.CollectionConfig:
        return grpc.CollectionConfig(
            params=cls.convert_collection_params(model.params),
            hnsw_config=cls.convert_hnsw_config(model.hnsw_config),
            optimizer_config=cls.convert_optimizers_config(model.optimizer_config),
            wal_config=cls.convert_wal_config(model.wal_config)
        )

    @classmethod
    def convert_hnsw_config(cls, model: rest.HnswConfig) -> grpc.HnswConfigDiff:
        return grpc.HnswConfigDiff(
            ef_construct=model.ef_construct,
            full_scan_threshold=model.full_scan_threshold,
            m=model.m,
        )

    @classmethod
    def convert_wal_config(cls, model: rest.WalConfig) -> grpc.WalConfigDiff:
        return grpc.WalConfigDiff(
            wal_capacity_mb=model.wal_capacity_mb,
            wal_segments_ahead=model.wal_segments_ahead
        )

    @classmethod
    def convert_distance(cls, model: rest.Distance) -> grpc.Distance:
        if model == rest.Distance.DOT:
            return grpc.Distance.Dot
        if model == rest.Distance.COSINE:
            return grpc.Distance.Cosine
        if model == rest.Distance.EUCLID:
            return grpc.Distance.Euclid

        raise ValueError(f"invalid Distance model: {model}")  # pragma: no cover

    @classmethod
    def convert_collection_params(cls, model: rest.CollectionParams) -> grpc.CollectionParams:
        return grpc.CollectionParams(
            vector_size=model.vector_size,
            shard_number=model.shard_number,
            distance=cls.convert_distance(model.distance)
        )

    @classmethod
    def convert_optimizers_config(cls, model: rest.OptimizersConfig) -> grpc.OptimizersConfigDiff:
        return grpc.OptimizersConfigDiff(
            default_segment_number=model.default_segment_number,
            deleted_threshold=model.deleted_threshold,
            flush_interval_sec=model.flush_interval_sec,
            indexing_threshold=model.indexing_threshold,
            max_optimization_threads=model.max_optimization_threads,
            max_segment_size=model.max_segment_size,
            memmap_threshold=model.memmap_threshold,
            vacuum_min_vector_number=model.vacuum_min_vector_number,
        )

    @classmethod
    def convert_optimizers_config_diff(cls, model: rest.OptimizersConfigDiff) -> grpc.OptimizersConfigDiff:
        return grpc.OptimizersConfigDiff(
            default_segment_number=model.default_segment_number,
            deleted_threshold=model.deleted_threshold,
            flush_interval_sec=model.flush_interval_sec,
            indexing_threshold=model.indexing_threshold,
            max_optimization_threads=model.max_optimization_threads,
            max_segment_size=model.max_segment_size,
            memmap_threshold=model.memmap_threshold,
            vacuum_min_vector_number=model.vacuum_min_vector_number,
        )

    @classmethod
    def convert_update_collection(cls, model: rest.UpdateCollection, collection_name: str) -> grpc.UpdateCollection:
        return grpc.UpdateCollection(
            collection_name=collection_name,
            optimizers_config=cls.convert_optimizers_config_diff(
                model.optimizers_config) if model.optimizers_config is not None else None
        )

    @classmethod
    def convert_geo_point(cls, model: rest.GeoPoint) -> grpc.GeoPoint:
        return grpc.GeoPoint(
            lon=model.lon,
            lat=model.lat
        )

    @classmethod
    def convert_match(cls, model: rest.Match) -> grpc.Match:
        if isinstance(model, rest.MatchValue):
            if isinstance(model.value, bool):
                return grpc.Match(boolean=model.value)
            if isinstance(model.value, int):
                return grpc.Match(integer=model.value)
            if isinstance(model.value, str):
                return grpc.Match(keyword=model.value)
        if isinstance(model, rest.MatchKeyword):
            return grpc.Match(keyword=model.keyword)
        if isinstance(model, rest.MatchInteger):
            return grpc.Match(integer=model.integer)

        raise ValueError(f"invalid Match model: {model}")  # pragma: no cover

    @classmethod
    def convert_alias_operations(cls, model: rest.AliasOperations) -> grpc.AliasOperations:
        if isinstance(model, rest.CreateAliasOperation):
            return grpc.AliasOperations(create_alias=cls.convert_create_alias(model.create_alias))
        if isinstance(model, rest.DeleteAliasOperation):
            return grpc.AliasOperations(delete_alias=cls.convert_delete_alias(model.delete_alias))
        if isinstance(model, rest.RenameAliasOperation):
            return grpc.AliasOperations(rename_alias=cls.convert_rename_alias(model.rename_alias))

        raise ValueError(f"invalid AliasOperations model: {model}")  # pragma: no cover

    @classmethod
    def convert_extended_point_id(cls, model: rest.ExtendedPointId) -> grpc.PointId:
        if isinstance(model, int):
            return grpc.PointId(num=model)
        if isinstance(model, str):
            return grpc.PointId(uuid=model)
        raise ValueError(f"invalid ExtendedPointId model: {model}")  # pragma: no cover

    @classmethod
    def convert_points_selector(cls, model: rest.PointsSelector) -> grpc.PointsSelector:
        if isinstance(model, rest.PointIdsList):
            return grpc.PointsSelector(
                points=grpc.PointsIdsList(ids=[cls.convert_extended_point_id(point) for point in model.points])
            )
        if isinstance(model, rest.FilterSelector):
            return grpc.PointsSelector(
                filter=cls.convert_filter(model.filter)
            )
        raise ValueError(f"invalid PointsSelector model: {model}")  # pragma: no cover

    @classmethod
    def convert_condition(cls, model: rest.Condition) -> grpc.Condition:
        if isinstance(model, rest.FieldCondition):
            return grpc.Condition(field=cls.convert_field_condition(model))
        if isinstance(model, rest.IsEmptyCondition):
            return grpc.Condition(is_empty=cls.convert_is_empty_condition(model))
        if isinstance(model, rest.HasIdCondition):
            return grpc.Condition(has_id=cls.convert_has_id_condition(model))
        if isinstance(model, rest.Filter):
            return grpc.Condition(filter=cls.convert_filter(model))

        raise ValueError(f"invalid Condition model: {model}")  # pragma: no cover

    @classmethod
    def convert_payload_selector(cls, model: rest.PayloadSelector) -> grpc.WithPayloadSelector:
        if isinstance(model, rest.PayloadSelectorInclude):
            return grpc.WithPayloadSelector(
                include=grpc.PayloadIncludeSelector(fields=model.include)
            )
        if isinstance(model, rest.PayloadSelectorExclude):
            return grpc.WithPayloadSelector(
                exclude=grpc.PayloadExcludeSelector(fields=model.exclude)
            )
        raise ValueError(f"invalid PayloadSelector model: {model}")  # pragma: no cover

    @classmethod
    def convert_with_payload_selector(cls, model: rest.PayloadSelector) -> grpc.WithPayloadSelector:
        return cls.convert_with_payload_interface(model)

    @classmethod
    def convert_with_payload_interface(cls, model: rest.WithPayloadInterface) -> grpc.WithPayloadSelector:
        if isinstance(model, bool):
            return grpc.WithPayloadSelector(enable=model)
        elif isinstance(model, list):
            return grpc.WithPayloadSelector(include=grpc.PayloadIncludeSelector(model))
        elif isinstance(model, (
                rest.PayloadSelectorInclude,
                rest.PayloadSelectorExclude,
        )):
            return cls.convert_payload_selector(model)

        raise ValueError(f"invalid WithPayloadInterface model: {model}")  # pragma: no cover

    @classmethod
    def convert_record(cls, model: rest.Record) -> grpc.RetrievedPoint:
        return grpc.RetrievedPoint(
            id=cls.convert_extended_point_id(model.id),
            payload=cls.convert_payload(model.payload) if model.payload is not None else None,
            vector=model.vector
        )

    @classmethod
    def convert_retrieved_point(cls, model: rest.Record) -> grpc.RetrievedPoint:
        return cls.convert_record(model)

    @classmethod
    def convert_count_result(cls, model: rest.CountResult) -> grpc.CountResult:
        return grpc.CountResult(count=model.count)

    @classmethod
    def convert_snapshot_description(cls, model: rest.SnapshotDescription) -> grpc.SnapshotDescription:
        return grpc.SnapshotDescription(
            name=model.name,
            creation_time=datetime.fromisoformat(model.creation_time),
            size=model.size,
        )



import os
from enum import Enum
from multiprocessing import Queue, Value, get_context
from multiprocessing.context import BaseContext
from multiprocessing.process import BaseProcess
from queue import Empty
from typing import Iterable, Any, Type, Optional, List

from loguru import logger

# Single item should be processed in less than:
processing_timeout = 10 * 60  # seconds

max_internal_batch_size = 200


class QueueSignals(str, Enum):
    stop = "stop"
    confirm = "confirm"
    error = "error"


class Worker:
    @classmethod
    def start(cls, **kwargs) -> 'Worker':
        raise NotImplementedError()

    def process(self, items: Iterable[Any]) -> Iterable[Any]:
        raise NotImplementedError()


def _worker(worker_class: Type[Worker],
            input_queue: Queue,
            output_queue: Queue,
            num_active_workers: Value,
            worker_id: int,
            kwargs=None) -> None:
    """
    A worker that pulls data pints off the input queue, and places the execution result on the output queue.
    When there are no data pints left on the input queue, it decrements
    num_active_workers to signal completion.
    """

    if kwargs is None:
        kwargs = {}

    logger.info(f"Reader worker: {worker_id} PID: {os.getpid()}")
    try:
        worker = worker_class.start(**kwargs)

        # Keep going until you get an item that's None.
        def input_queue_iterable() -> Iterable[Any]:
            while True:
                item = input_queue.get()
                if item == QueueSignals.stop:
                    break
                yield item

        for processed_item in worker.process(input_queue_iterable()):
            output_queue.put(processed_item)
    except Exception as e:  # pylint: disable=broad-except
        logger.exception(e)
        output_queue.put(QueueSignals.error)
    finally:
        # It's important that we close and join the queue here before
        # decrementing num_active_workers. Otherwise our parent may join us
        # before the queue's feeder thread has passed all buffered items to
        # the underlying pipe resulting in a deadlock.
        #
        # See:
        # https://docs.python.org/3.6/library/multiprocessing.html?highlight=process#pipes-and-queues
        # https://docs.python.org/3.6/library/multiprocessing.html?highlight=process#programming-guidelines
        output_queue.close()
        output_queue.join_thread()

        with num_active_workers.get_lock():
            num_active_workers.value -= 1

        logger.info(f"Reader worker {worker_id} finished")


class ParallelWorkerPool:

    def __init__(self, num_workers, worker: Type[Worker], start_method=None):
        self.worker_class = worker
        self.num_workers = num_workers
        self.input_queue: Optional[Queue] = None
        self.output_queue: Optional[Queue] = None
        self.ctx: Optional[BaseContext] = get_context(start_method)
        self.processes: List[BaseProcess] = []
        self.queue_size = self.num_workers * max_internal_batch_size

        self.num_active_workers: Optional[Value] = None

    def start(self, **kwargs):
        self.input_queue = self.ctx.Queue(self.queue_size)
        self.output_queue = self.ctx.Queue(self.queue_size)

        self.num_active_workers = self.ctx.Value('i', self.num_workers)

        for worker_id in range(0, self.num_workers):
            process = self.ctx.Process(
                target=_worker,
                args=(
                    self.worker_class,
                    self.input_queue,
                    self.output_queue,
                    self.num_active_workers,
                    worker_id,
                    kwargs.copy())
            )
            process.start()
            self.processes.append(process)

    def unordered_map(self, stream: Iterable[Any], *args, **kwargs) -> Iterable[Any]:
        try:
            self.start(**kwargs)
            pushed = 0
            read = 0
            for item in stream:

                if pushed - read < self.queue_size:
                    try:
                        out_item = self.output_queue.get_nowait()
                    except Empty:
                        out_item = None
                else:
                    try:
                        out_item = self.output_queue.get(timeout=processing_timeout)
                    except Empty as e:
                        self.join_or_terminate()
                        raise e

                if out_item is not None:
                    if out_item == QueueSignals.error:
                        self.join_or_terminate()
                        raise RuntimeError("Thread unexpectedly terminated")
                    yield out_item
                    read += 1

                self.input_queue.put(item)
                pushed += 1

            for _ in range(self.num_workers):
                self.input_queue.put(QueueSignals.stop)

            while read < pushed:
                out_item = self.output_queue.get(timeout=processing_timeout)
                if out_item == QueueSignals.error:
                    self.join_or_terminate()
                    raise RuntimeError("Thread unexpectedly terminated")
                yield out_item
                read += 1
        finally:
            self.input_queue.close()
            self.output_queue.close()

    def join_or_terminate(self, timeout=1):
        """
        Emergency shutdown
        @param timeout:
        @return:
        """
        for process in self.processes:
            process.join(timeout=timeout)
            if process.is_alive():
                process.terminate()
        self.processes.clear()

    def join(self) -> None:
        for process in self.processes:
            process.join()
        self.processes.clear()

    def __del__(self) -> None:
        """
        Terminate processes if the user hasn't joined. This is necessary as
        leaving stray processes running can corrupt shared state. In brief,
        we've observed shared memory counters being reused (when the memory was
        free from the perspective of the parent process) while the stray
        workers still held a reference to them.
        For a discussion of using destructors in Python in this manner, see
        https://eli.thegreenplace.net/2009/06/12/safely-using-destructors-in-python/.
        """
        for process in self.processes:
            process.terminate()



from typing import Iterable, Any

from nnext.http import SyncApis
from nnext.http.models import PointsBatch, Batch
from nnext.uploader.uploader import BaseUploader


def upload_batch(openapi_client: SyncApis, collection_name: str, batch) -> bool:
    ids_batch, vectors_batch, payload_batch = batch

    # Make sure we do not send too many ids in case there is an iterable over vectors,
    # and we do not know how many ids are required in advance
    if len(ids_batch) > len(vectors_batch):
        ids_batch = ids_batch[:len(vectors_batch)]

    if payload_batch is not None:
        payload_batch = list(payload_batch)

    openapi_client.points_api.upsert_points(
        collection_name=collection_name,
        point_insert_operations=PointsBatch(
            batch=Batch(
                ids=ids_batch,
                payloads=payload_batch,
                vectors=vectors_batch
            )
        )
    )
    return True


class RestBatchUploader(BaseUploader):

    def __init__(self, uri, collection_name, **kwargs: Any):
        self.collection_name = collection_name
        self.openapi_client = SyncApis(host=uri, **kwargs)

    @classmethod
    def start(cls, collection_name=None, uri="http://localhost:6333", **kwargs) -> 'RestBatchUploader':
        if not collection_name:
            raise RuntimeError("Collection name could not be empty")
        return cls(uri=uri, collection_name=collection_name, **kwargs)

    def process(self, items: Iterable[Any]) -> Iterable[Any]:
        for batch in items:
            yield upload_batch(self.openapi_client, self.collection_name, batch)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nnext',
 'nnext.conversions',
 'nnext.grpc',
 'nnext.http',
 'nnext.http.api',
 'nnext.http.models',
 'nnext.uploader']

package_data = \
{'': ['*']}

install_requires = \
['betterproto==2.0.0b4',
 'grpcio>=1.46.0,<2.0.0',
 'httpx[http2]>=0.23.0,<0.24.0',
 'loguru>=0.5.3,<0.6.0',
 'nest-asyncio>=1.5.5,<2.0.0',
 'numpy>=1.21,<2.0',
 'pydantic>=1.8,<2.0',
 'tqdm>=4.56.0,<5.0.0',
 'typing-extensions>=4.0.0,<5.0.0']

setup_kwargs = {
    'name': 'nnext',
    'version': '0.0.39',
    'description': 'Client library for the NNext vector search engine',
    'long_description': '\n# <a href="https://nnext.ai/"><img src="https://d135j1zm1liera.cloudfront.net/nnext-logo-wide.png" height="100" alt="Apollo Client"></a>\n\n## About\n\nThis repository houses the source code for the python client associated with NNext.\n\nNNext is a\n* ⚡ blazingly fast\n* 🔍 nearest-neighbors vector search engine\n\n<a href="https://tiny.one/nnext-slk-comm-gh"><img src="https://img.shields.io/badge/chat-slack-orange.svg?logo=slack&style=flat"></a>\n<a href="https://twitter.com/intent/follow?screen_name=nnextai"><img src="https://img.shields.io/badge/Follow-nnextai-blue.svg?style=flat&logo=twitter"></a>\n\n[Installation](#installation) |  [Quick Start](#quick-start) | [Documentation](#documentation)\n\n## Installation\nTo install the pynnext client, activate a virtual environment, and install via pip:\n\n```zsh\npip install nnext\n```\n\n## Quick Start\n\nHere\'s a quick example showcasing how you can create an index, insert vectors/documents and search among them via NNext.\n\nLet\'s begin by installing the NNext server.\n\n```zsh\ndocker run -p 6040:6040 -p 6041:6041 \\\n\t-v ~/.nnext/data:/.nnext/data \\\n\tnnext/nnext\n```\n\n## Documentation\nMore documentation is available here:\n\nhttps://nnext.ai/docs',
    'author': 'NNext Team',
    'author_email': 'pynnext@nnext.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/nnextdb/pynnext',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import asyncio
from itertools import count
from typing import Iterable, Any

from grpclib.client import Channel

from nnext.conversions.conversion import payload_to_grpc, RestToGrpc
from nnext.grpc import PointsStub, PointStruct, PointId
from nnext.uploader.uploader import BaseUploader


def iter_over_async(ait, loop):
    ait = ait.__aiter__()

    async def get_next():
        try:
            obj = await ait.__anext__()
            return False, obj
        except StopAsyncIteration:
            return True, None

    while True:
        done, obj = loop.run_until_complete(get_next())
        if done:
            break
        yield obj


async def upload_batch_grpc(points_client: PointsStub, collection_name: str, batch):
    ids_batch, vectors_batch, payload_batch = batch
    if payload_batch is None:
        payload_batch = (None for _ in count())

    points = [
        PointStruct(
            id=RestToGrpc.convert_extended_point_id(idx) if not isinstance(idx, PointId) else idx,
            vector=vector,
            payload=payload_to_grpc(payload or {}),
        ) for idx, vector, payload in zip(ids_batch, vectors_batch, payload_batch)
    ]
    return await points_client.upsert(collection_name=collection_name, points=points)


class GrpcBatchUploader(BaseUploader):

    def __init__(self, host, port, collection_name, metadata=None, **kwargs):
        self.collection_name = collection_name
        self._host = host
        self._port = port
        self._kwargs = kwargs
        self._metadata = metadata

    @classmethod
    def start(cls, collection_name=None, host="localhost", port=6334, **kwargs) -> 'GrpcBatchUploader':
        if not collection_name:
            raise RuntimeError("Collection name could not be empty")

        return cls(host=host, port=port, collection_name=collection_name, **kwargs)

    async def process_upload(self, items):
        async with Channel(host=self._host, port=self._port, **self._kwargs) as channel:
            points_client = PointsStub(channel, metadata=self._metadata)
            for batch in items:
                yield await upload_batch_grpc(points_client, self.collection_name, batch)

    def process(self, items: Iterable[Any]) -> Iterable[Any]:
        loop = asyncio.get_event_loop()
        async_gen = self.process_upload(items)
        sync_gen = iter_over_async(async_gen, loop)
        return sync_gen
